import React from 'react';
import ReactDOM from 'react-dom';
import AdainTree, {toggleExpandedForAll} from 'react-adain-tree';
import { MdAssignment, MdFlashOn, MdContentCopy, MdDelete, MdVerticalAlignTop, MdVerticalAlignBottom } from 'react-icons/lib/md';
import _ from 'lodash';

class App extends React.Component {
	constructor(props) {
		super(props);
		
		let treeData = [
			{_id: 'a', expanded: true, name: 'A', hasCheckBox: true, status: 'statustest', assign1: 'assign1', assign2: 'assign2', assign3: 'assign3', isFolder: true, children:[
				{_id: 'aa', expanded: true, name: 'AA', assignedUserName: 'mgkim'}, {_id: 'ab', name: 'AB', assignedUserName: 'syjang'}, {_id: 'ac', name: 'AC', isFolder: true, children: [
					{_id: 'aca', name: 'ACA', assignedUserName: 'yes'}, {name: 'ACB', assignedUserName: 'mgkim'}, {name: 'ACC', assignedUserName: 'acc'}
				]}
			]},
			{_id: 'b', name: 'B', assignedUserName: 'mgkim'},
			{_id: 'c', name: 'C', isFolder: true, children:[
				{_id: 'ca', name: 'CA', assignedUserName: 'mgkim'}, {_id: 'cb', name: 'CB', assignedUserName: 'good'}
			]},
			{_id: 'd', name: 'D', assignedUserName: 'mgkim', isFolder: true, children: []},
		];

		const self = this;
		const makeAssignFn = (item) => {
			if (!_.isEmpty(item.assignedUserName)) {
				item.assign3 = <button
					className="btnNodeName"
					style={{
						verticalAlign: 'middle',
					}}
					onClick={() => self.onClickNodeName(item)}
					title="Name"
				>
					{item.assignedUserName}
				</button>
			}
		};
		
		const traverse = (item) => {
			makeAssignFn(item);
			if (item.children && item.children.length > 0) {
				item.children.map((itemChildren) => {
					traverse(itemChildren);
				})
			}
		};
		
		treeData.map((item) => {
			traverse(item);
		});
		
		this.state = {
			searchString: '',
			searchFocusIndex: 0,
			searchFoundCount: null,
			treeData: treeData
		};

		this.updateTreeData = this.updateTreeData.bind(this);
		this.search = this.search.bind(this);
		this.expandAll = this.expandAll.bind(this);
		this.collapseAll = this.collapseAll.bind(this);
		this.onClickNodeName = this.onClickNodeName.bind(this);
		this.onClickCopy = this.onClickCopy.bind(this);
		this.onClickDelete = this.onClickDelete.bind(this);

		this.onRowClick = this.onRowClick.bind(this);
	}
	updateTreeData(treeData) {
		this.setState({ treeData });
	}

	expand(expanded) {
		this.setState({
			treeData: toggleExpandedForAll({
				treeData: this.state.treeData,
				expanded,
			}),
		});
	}

	expandAll() {
		this.expand(true);
	}

	collapseAll() {
		this.expand(false);
	}

	search() {
		console.log(this.refs.txtSearch.value);
	}

	render () {
		const {
			treeData,
			searchString,
			searchFocusIndex,
			searchFoundCount,
		} = this.state;

		return (
			<div>
				<div>
					<form
						style={{ display: 'inline-block' }}
						onSubmit={event => {
							event.preventDefault();
						}}
					>
						<label htmlFor="find-box">
							Search:&nbsp;

							<input
								id="find-box"
								type="text"
								value={searchString}
								onChange={event => this.setState({ searchString: event.target.value })}
							/>
						</label>
					</form>
				</div>
				<AdainTree childPropertyName="children"
									 titleField="name"
									 commonSetting={
										 {
											 hasCheckBoxAll: true
										 }
									 }
									 divHeight="350"
									 rowHeight={49}
									 scaffoldBlockPxWidth={30}
									 header={[
										 {field: 'name', title: 'TITLE', hasSort: true, isTitle: true, className: 'treeHeaderRowTitle'},
										 {field: 'status', title: 'STATUS', width: '12%', className: 'treeHeaderRowItem'},
										 {field: 'assign1', title: 'ASSIGN1', width: '12%', className: 'treeHeaderRowItem'},
										 {field: 'assign2', title: 'ASSIGN2', width: '12%', className: 'treeHeaderRowItem'},
										 {field: 'assign3', title: 'ASSIGN3', width: '12%', className: 'treeHeaderRowItem'},
										 {field: 'buttons', title: '', width: '20%', hasButtons: true, className: 'treeHeaderRowButtons'},
									 ]}
									 treeData={treeData}
									 onChange={this.updateTreeData}
									 onRowClick={this.onRowClick}
									 onMoveNode={({ node, treeIndex, path }) =>
										 console.debug(
											 'node:', node,
											 'treeIndex:', treeIndex,
											 'path:', path
										 )}
									 onVisibilityToggle={({treeData, node, expanded}) => {
										 console.log(treeData, node, expanded)
									 }}
									 canDrag={({ node }) => !node.noDragging}
									 canDrop={({ nextParent }) =>
									 !nextParent || !nextParent.noChildren}
									 searchQuery={searchString}
									 searchFocusOffset={searchFocusIndex}
									 searchFinishCallback={matches =>
										 this.setState({
											 searchFoundCount: matches.length,
											 searchFocusIndex: matches.length > 0
												 ? searchFocusIndex % matches.length
												 : 0,
										 })}
									 generateNodeProps={(rowInfo)=>{
										 if(rowInfo.node.children){
											 return ({buttons: [
												 <button
													 className="btnNodeBlank"
												 >
													 &nbsp;
												 </button>,
												 <button
													 className="btnNodeBlank"
												 >
													 &nbsp;
												 </button>,
												 <button
													 className="btnNodeCopy"
													 title="Copy"
													 onClick={() => this.onClickCopy(rowInfo)}
												 >
													 <MdContentCopy />
												 </button>,
												 <button
													 className="btnNodeDelete"
													 title="Delete"
													 onClick={() => this.onClickDelete(rowInfo)}
												 >
													 <MdDelete />
												 </button>
											 ]});
										 }else{
											 return ({
												 buttons: [
													 <button
														 className="btnNodeAssignment disabled"
														 title="Assign"
													 >
														 <MdAssignment />
													 </button>,
													 <button
														 className="btnNodeFlashOn disabled"
														 title="FlashOn"
													 >
														 <MdFlashOn />
													 </button>,
													 <button
														 className="btnNodeCopy"
														 title="Copy"
														 onClick={() => this.onClickCopy(rowInfo)}
													 >
														 <MdContentCopy />
													 </button>,
													 <button
														 className="btnNodeDelete"
														 title="Delete"
														 onClick={() => this.onClickDelete(rowInfo)}
													 >
														 <MdDelete />
													 </button>
												 ],
											 });
										 }
									 }}
									 type="static"  // static | dynamic
				/>
			</div>
		);
	}

	onRowClick({node, path, treeIndex}) {
		console.log('on click row', node, path, treeIndex);
	}

	onClickNodeName(node) {
		
		console.log('on click node name', node);
	}

	onClickCopy({node, path, treeIndex}) {
		console.log('on click copy', node, path, treeIndex);
	}

	onClickDelete({node, path, treeIndex}) {
		console.log('on click delete', node, path, treeIndex);
	}
}

ReactDOM.render(<App />, document.getElementById('app'));
